
#include "pgrutils.h"

// 1394 Driver header files
#include <dc1394/utils.h>
#include <dc1394/control.h>
#include <dc1394/conversions.h>

//#include <libplayerc/playerc.h>
//#include <libplayerc++/playerc++.h>

// Qt header files. Used to store the images
//#include <QtGui/qpainter.h>
//#include <QtGui/qpixmap.h>
//#include <QtGui/qwidget.h>
//#include <QtGui/qwmatrix.h>
//#include <Qt/qtextstream.h>
//#include <QtGui/qimage.h>

#include "camClass.h"

#define PI 3.14159265


CamClass::CamClass()
{

    //////////////////////////////////////
    // Initialize some pointers to NULL //
    //////////////////////////////////////
    
    cameras = NULL;
    sony_camera=NULL;
    df_camera[0]=NULL;
    df_camera[1]=NULL;
    df2_camera[0]=NULL;
    df2_camera[1]=NULL;
    
    //////////////////////////////////////
    // Setting some global variables	//
    //////////////////////////////////////
//	sony_GUID=((uint64_t)0x08004602<<32)+0x0004008b;
	df_GUID[0]=((uint64_t)0x00b09d01<<32)+0x003f0618;
	df_GUID[1]=((uint64_t)0x00b09d01<<32)+0x003f0619;
	df2_GUID[0]=((uint64_t)0x00b09d01<<32)+0x006d7145;
	df2_GUID[1]=((uint64_t)0x00b09d01<<32)+0x006d7142;
    
    dma_channels=2;

    frameNum=0;
        
//    sonyStr= string("xcd_im_");
    dfStr[0]=string("df_left_im_");
    dfStr[1]=string("df_right_im_");
    df2Str[0]=string("df2_left_im_");
    df2Str[1]=string("df2_right_im_");
}

CamClass::~CamClass()
{
    cout << "cleaning up" << endl;
    cleanup();
}


dc1394error_t CamClass::init()
{
    //////////////////
    // Find cameras //
    //////////////////
    dc1394error_t err=dc1394_find_cameras(&cameras, &numCameras);
    if (err!=DC1394_SUCCESS && err != DC1394_NO_CAMERA) {
        cerr << "Unable to look for cameras" << endl << endl;
        cerr << "On Linux, please check" << endl;
        cerr << "  - if the kernel modules `ieee1394',`raw1394' and `ohci1394' are loaded" << endl;
        cerr << "  - if you have read/write access to /dev/raw1394" << endl << endl;
        return DC1394_SUCCESS;
    }
    
    //////////////////////////////////////////////////////////////////////////////////
    // get the camera nodes and identify the dragonfly, dragonfly2 and sony cameras	//
    //////////////////////////////////////////////////////////////////////////////////
    if (numCameras<1) {
        cerr << "no cameras found" << endl;
        exit(1);
    }
//    cout << "Found " << numCameras << " cameras on the 1394 bus" << endl;

    // Store the camera handles in the proper structure. To do that we should compare the GUID of the cameras
    for (unsigned int i=0; i<numCameras; i++)
    {
//        if ((cameras[i]->euid_64)==sony_GUID) sony_camera=cameras[i];
        if ((cameras[i]->euid_64)==df_GUID[0]) df_camera[0]=cameras[i];
        else if ((cameras[i]->euid_64)==df_GUID[1]) df_camera[1]=cameras[i];
        else if ((cameras[i]->euid_64)==df2_GUID[0]) df2_camera[0]=cameras[i];
        else if ((cameras[i]->euid_64)==df2_GUID[1]) df2_camera[1]=cameras[i];
    } 
    
    // Check if we found all the cameras
//    if (sony_camera==NULL) return DC1394_NO_CAMERA;
    if (df_camera[0]==NULL) return DC1394_NO_CAMERA;
    if (df_camera[1]==NULL) return DC1394_NO_CAMERA;
    if (df2_camera[0]==NULL) return DC1394_NO_CAMERA;
    if (df2_camera[1]==NULL) return DC1394_NO_CAMERA;
    
//    cout << "Found the cameras" << endl;
	
	// Check that the cameras have the one-shot capability  //
	// Appears that only the SONY camera does NOT support the ONE-SHOT FUNCTION
	// Both DragonFly and DragonFly2 cameras SUPPORT ONE-SHOT FUNCTION
    //cout << "Sony: " << sony_camera->one_shot_capable << " DF " << df_camera[0]->one_shot_capable << " DF " <<  df_camera[1]->one_shot_capable << " DF2 " << df2_camera[0]->one_shot_capable << " DF2 " <<df_camera[1]->one_shot_capable << endl;
	
    //////////////////////////////////////////////
    // Read the register file and store the     //
    // values to the appropriate data structure //
    //////////////////////////////////////////////
    readRegisterFileDF("df_registerValues.txt",df_registers);
    readRegisterFileDF2("df2_registerValues.txt",df2_registers);
    //printRegisterFile(df2_registers);

	//////////////////////////////////////////
    // Obtain the Bayer Pattern in advance	//
    //////////////////////////////////////////
    err=getBayerTile( df_camera[0], &bayerPattern );
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not get bayer tile pattern from left camera" << endl;
        return err;
    }

    // Obtain the frame size for all the cameras in advance
    dc1394_get_image_size_from_video_mode( sony_camera, DC1394_VIDEO_MODE_1024x768_MONO8, &sony_width, &sony_height );    
//    cout << "Sony XCD-X700: Image size :" << df_width << "x" << df_height << endl;
    dc1394_get_image_size_from_video_mode( df_camera[0], DC1394_VIDEO_MODE_1024x768_MONO8, &df_width, &df_height );    
//    cout << "Dragon Fly: Image size :" << df_width << "x" << df_height << endl;
    dc1394_get_image_size_from_video_mode( df2_camera[0], DC1394_VIDEO_MODE_1024x768_RGB8, &df2_width, &df2_height );    
//    cout << "Dragon Fly2: Image size :" << df2_width << "x" << df2_height << endl;
    return DC1394_SUCCESS;
}

dc1394error_t CamClass::setAllCameraRegisters(unsigned int a_uSleepTime)
{
	dc1394error_t err;
	
/*	err = setSonyCameraRegisters(sony_camera, dma_channels);
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not set Sony Camera Registers" << endl;
        return err;
    }
    cout << ". ";	
    usleep(a_uSleepTime);
*/	err = setDragonFlyCameraRegisters(df_camera[0], 1, df_registers); // Iso channel 1
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not set first DragonFly Camera Registers" << endl;
        return err;
    }
    cout << ". ";	
    usleep(a_uSleepTime);
	err = setDragonFlyCameraRegisters(df_camera[1], 2, df_registers); // Iso channel 2
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not set second DragonFly Camera Registers" << endl;
        return err;
    }
    cout << ". ";	
    usleep(a_uSleepTime);
	err = setDragonFly2CameraRegisters(df2_camera[0], 3, df2_registers); // Iso channel 3
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not set first DragonFly2 Camera Registers" << endl;
        return err;
    }
    cout << ". ";	
    usleep(a_uSleepTime);
	err = setDragonFly2CameraRegisters(df2_camera[1], 4, df2_registers); // Iso channel 4
    if ( err != DC1394_SUCCESS )
    {
        cerr << "Could not set second DragonFly2 Camera Registers" << endl;
        return err;
    }
    usleep(a_uSleepTime);
    cout << ". " << endl;
	return DC1394_SUCCESS;
}

dc1394error_t CamClass::triggerAllCameras()
{
	dc1394error_t err;	
	
	// Use the ONE-PUSH MODE FOR THE CAMERAS
    err=dc1394_video_set_one_shot(df_camera[0], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start left dragon fly camera iso transmission" << endl;
        return err;
    }
	
    err=dc1394_video_set_one_shot(df_camera[1], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start right dragon fly camera iso transmission" << endl;
        return err;
    }

    err=dc1394_video_set_one_shot(df2_camera[0], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start left dragon fly camera iso transmission" << endl;
        return err;
    }

    err=dc1394_video_set_one_shot(df2_camera[1], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start right dragon fly camera iso transmission" << endl;
        return err;
    }

	return DC1394_SUCCESS;

    //////////////////////////////
    // start all the cameras    //
    //////////////////////////////
/*    // Sony XCD-X700
    err=dc1394_video_set_transmission(sony_camera, DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start sony camera iso transmission" << endl;
        return err;
    }
*/

/*    // DragonFly
    err=dc1394_video_set_transmission(df_camera[0], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start left dragon fly camera iso transmission" << endl;
        return err;
    }
    err=dc1394_video_set_transmission(df_camera[1], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start right dragon fly camera iso transmission" << endl;
        return err;
    }

    err=dc1394_video_set_transmission(df2_camera[0], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start left dragon fly2 camera iso transmission" << endl;
        return err;
    }
    err=dc1394_video_set_transmission(df2_camera[1], DC1394_ON);
    if (err!=DC1394_SUCCESS) {
        cerr << "unable to start right dragon fly2 camera iso transmission" << endl;
        return err;
    }
*/
	//////////////////////////////
	// Verify the transmission	//
	//////////////////////////////		
	// Sony cameras
/*    err=verifyCameraTransmission(sony_camera);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "Sony Camera transmission function failed" << endl;
        return err;
    }        
	// Left DragonFly camera
    err=verifyCameraTransmission(df_camera[0]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "Left DragonFly Camera transmission function failed" << endl;
        return err;
    }        
	// Right DragonFly camera
    err=verifyCameraTransmission(df_camera[1]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "Right DragonFly Camera transmission function failed" << endl;
        return err;
    }        
	// Left DragonFly 2 camera
    err=verifyCameraTransmission(df2_camera[0]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "Left DragonFly2 Camera transmission function failed" << endl;
        return err;
    }        
	// Right DragonFly 2 camera		
    err=verifyCameraTransmission(df2_camera[1]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "Right DragonFly2 Camera transmission function failed" << endl;
        return err;
    }        
	return DC1394_SUCCESS;
*/
}


dc1394error_t CamClass::captureSetup()
{
    dc1394error_t cur_status;
    
    //////////////////////
    // SETUP CAPTURE    //
    //////////////////////
    // Left DragonFly
    cur_status=dc1394_capture_setup(df_camera[0], dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
    // Right DragonFly
    cur_status=dc1394_capture_setup(df_camera[1], dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
    // Left DragonFly2
    cur_status=dc1394_capture_setup(df2_camera[0], dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
    // Right DragonFly2
    cur_status=dc1394_capture_setup(df2_camera[1], dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
	return DC1394_SUCCESS;
}

dc1394error_t CamClass::captureAllFrames()
{
	dc1394error_t err;

    // Empty all the DMA channels
//    empty_dma_channels(sony_camera, dma_channels); 
/*    empty_dma_channels(df_camera[0], dma_channels); 
    empty_dma_channels(df_camera[1], dma_channels); 
    empty_dma_channels(df2_camera[0], dma_channels);
    empty_dma_channels(df2_camera[1], dma_channels);
*/   
    //////////////////////////////////////////    
    // Obtain the frames from all cameras //
    //////////////////////////////////////////
/*    err=dc1394_capture_dequeue(sony_camera, DC1394_CAPTURE_POLICY_WAIT, &sony_frame);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "unable to capture a frame with Sony Camera" << endl;
		return err;
    }
*/
    cout << "[Dequeue] ";   
    err=dc1394_capture_dequeue(df_camera[0], DC1394_CAPTURE_POLICY_WAIT, &df_frame[0]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "unable to capture a frame with left DragonFly camera" << endl;
		return err;
    }
    cout << ". ";
    err=dc1394_capture_dequeue(df_camera[1], DC1394_CAPTURE_POLICY_WAIT, &df_frame[1]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "unable to capture a frame with Right DragonFly camera" << endl;
		return err;
    }
    cout << ". ";
    err=dc1394_capture_dequeue(df2_camera[0], DC1394_CAPTURE_POLICY_WAIT, &df2_frame[0]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "unable to capture a frame left DragonFly2 camera" << endl;
		return err;
    }
    cout << ". ";
    err=dc1394_capture_dequeue(df2_camera[1], DC1394_CAPTURE_POLICY_WAIT, &df2_frame[1]);
    if (err!=DC1394_SUCCESS)
    {
        cerr << "unable to capture a frame right DragonFly2 camera" << endl;
		return err;
    }
    cout << ". " << endl;

    cout << "Writing files to Files" << endl;
	// Write Frames to Files
	writeFramesToFiles();
	cout << "done" << endl;
	
    //////////////////////////////////////////////
    // Return all the buffers to the DMA pool   //
    ////////////////////////////////////////////// 
/*    err=dc1394_capture_enqueue(sony_camera, sony_frame);
    if (err!=DC1394_SUCCESS) 
    {
        cerr << "unable to enqueue the DMA buffer" << endl;
        return err;
    }
*/    err=dc1394_capture_enqueue(df_camera[0], df_frame[0]);
    if (err!=DC1394_SUCCESS) 
    {
        cerr << "unable to enqueue the DMA buffer" << endl;
        return err;
    }
    err=dc1394_capture_enqueue(df_camera[1], df_frame[1]);
    if (err!=DC1394_SUCCESS) 
    {
        cerr << "unable to enqueue the DMA buffer" << endl;
        return err;
    }
    err=dc1394_capture_enqueue(df2_camera[0], df2_frame[0]);
    if (err!=DC1394_SUCCESS) 
    {
        cerr << "unable to enqueue the DMA buffer" << endl;
        return err;
    }
    err=dc1394_capture_enqueue(df2_camera[1], df2_frame[1]);
    if (err!=DC1394_SUCCESS) 
    {
        cerr << "unable to enqueue the DMA buffer" << endl;
        return err;
    }
		
	return DC1394_SUCCESS;
}


bool CamClass::writeFramesToFiles( )
{
//	int res;		
	char filename[50];
	sprintf(filename, "%s%i.pgm", sonyStr.c_str(),frameNum);
	// Sony Cameras
	//res+=writeBWFrameToFile( filename, sony_width, sony_height, sony_frame);
	
	// DragonFly Cameras
	sprintf(filename, "%s%i.ppm", dfStr[0].c_str(),frameNum);
	writeRawFrameToFile( filename, df_width, df_height, df_frame[0], bayerPattern);
	sprintf(filename, "%s%i.ppm", dfStr[1].c_str(),frameNum);
	writeRawFrameToFile( filename, df_width, df_height, df_frame[1], bayerPattern);

	// DragonFly 2 Cameras
	sprintf(filename, "%s%i.ppm", df2Str[0].c_str(),frameNum);
	writeRGBFrameToFile( filename, df2_width, df2_height, df2_frame[0]);
	sprintf(filename, "%s%i.ppm", df2Str[1].c_str(),frameNum);
	writeRGBFrameToFile( filename, df2_width, df2_height, df2_frame[1]);
	
//	if (res<4)
//	{
//		cerr << "Couldn't write all frames to disk" << endl;
//		return false;
//	}
//	else 
    frameNum++;
	return true;		
}	


bool CamClass::writeBWFrameToFile( char *ap_filename, unsigned int a_width, unsigned int a_height, dc1394video_frame_t *ap_frame)
{
    FILE * imageFile=fopen( ap_filename, "w" );

    fprintf(imageFile,"P5\n%u %u 255\n", a_width, a_height);
    fwrite(ap_frame->image, 1, a_height*a_width, imageFile);
    fclose(imageFile);
    return true;
}

bool CamClass::writeRGBFrameToFile( char *ap_filename, unsigned int a_width, unsigned int a_height, dc1394video_frame_t *ap_frame)
{
    FILE * imageFile=fopen( ap_filename, "w" );

    // note: assuming a 24 bit image
    fprintf( imageFile, "P6\n%u %u 255\n", a_width, a_height );
    fwrite(ap_frame->image, 3, a_height*a_width, imageFile);
    fclose(imageFile);
    return true;
}

bool CamClass::writeRawFrameToFile( char *ap_filename, unsigned int a_width, unsigned int a_height, dc1394video_frame_t *ap_frame, dc1394color_filter_t &a_bayerPattern)
{
    FILE * imageFile=fopen( ap_filename, "w" );

    uint8_t* pRGBImage = new uint8_t[ a_width*a_height*3 ];


    dc1394_bayer_decoding_8bit( ap_frame->image, pRGBImage, a_width, a_height, a_bayerPattern, DC1394_BAYER_METHOD_NEAREST );

    // note: assuming a 24 bit image
    fprintf( imageFile, "P6\n%u %u 255\n", a_width, a_height );
    fwrite( pRGBImage, 3, a_width*a_height, imageFile );
    fclose( imageFile );

    return true;
}

void CamClass::printGUID(dc1394camera_t *ap_camera)
{
    using namespace std;
    cout << "GUID:" << ap_camera->euid_64 << "  VendorID:" << ap_camera->vendor_id << "  ModelID:" << ap_camera->model_id << endl;
    return;
}

/*dc1394error_t CamClass::setSonyCameraRegisters(dc1394camera_t *ap_camera, int a_dma_channels)
{
    dc1394error_t cur_status;
    using namespace std;
    
    // Turn the camera on
    cur_status=dc1394_set_camera_power(ap_camera, DC1394_ON);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to power up the camera" << endl;
        return cur_status;
    }
    
    // Isochronous speed of the camera
    cur_status=dc1394_video_set_iso_speed(ap_camera, DC1394_ISO_SPEED_400);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the isochronous speed of the camera" << endl;
        return cur_status;
    }

    // Set the operation mode to 1394A (instead of 1394B)    
    cur_status= dc1394_video_set_operation_mode(ap_camera, DC1394_OPERATION_MODE_LEGACY);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the operation mode of the camera" << endl;
        return cur_status;
    }
    
    // Set mode
    cur_status=dc1394_video_set_mode(ap_camera, DC1394_VIDEO_MODE_1024x768_MONO8);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the mode of the camera" << endl;
        return cur_status;
    }

    // Set framerate
    cur_status=dc1394_video_set_framerate(ap_camera, DC1394_FRAMERATE_7_5); // DC1394_FRAMERATE_15
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the framerate of the camera" << endl;
        return cur_status;
    }
    
    // Turn one-shot off
    cur_status=dc1394_video_set_one_shot(ap_camera, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn one-shot mode off" << endl;
        return cur_status;
    }

    // Turn multi-shot off
    cur_status=dc1394_video_set_multi_shot(ap_camera, 0, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn multi-shot mode off" << endl;
        return cur_status;
    }
    
    // Adjust the Gain feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_GAIN, 2048);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain value to 2048" << endl;
        return cur_status;
    }

    // Adjust the Shutter feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHUTTER, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SHUTTER, 2600);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter value to 2600" << endl;
        return cur_status;
    }

    // Setup the capture
    cur_status=dc1394_capture_setup(ap_camera, a_dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
    return DC1394_SUCCESS;
}
*/

dc1394error_t CamClass::setDragonFlyCameraRegisters(dc1394camera_t *ap_camera, int iso_channel, DF_REGS &reg_struct)
{
/*    dc1394error_t cur_status;
    using namespace std;
    
    // Turn the camera on
    cur_status=dc1394_set_camera_power(ap_camera, DC1394_ON);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to power up the camera" << endl;
        return cur_status;
    }
    
    // Isochronous speed of the camera
    cur_status=dc1394_video_set_iso_speed(ap_camera, DC1394_ISO_SPEED_400);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the isochronous speed of the camera" << endl;
        return cur_status;
    }

    // Set the operation mode to 1394A (instead of 1394B)    
    cur_status= dc1394_video_set_operation_mode(ap_camera, DC1394_OPERATION_MODE_LEGACY);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the operation mode of the camera" << endl;
        return cur_status;
    }
    
    // Set mode
    cur_status=dc1394_video_set_mode(ap_camera, DC1394_VIDEO_MODE_1024x768_MONO8);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the mode of the camera" << endl;
        return cur_status;
    }

    // Set framerate
    cur_status=dc1394_video_set_framerate(ap_camera, DC1394_FRAMERATE_1_875); //DC1394_FRAMERATE_3_75, DC1394_FRAMERATE_7_5,DC1394_FRAMERATE_15
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the framerate of the camera" << endl;
        return cur_status;
    }
    
    // Turn one-shot off
    cur_status=dc1394_video_set_one_shot(ap_camera, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn one-shot mode off" << endl;
        return cur_status;
    }

    // Turn multi-shot off
    cur_status=dc1394_video_set_multi_shot(ap_camera, 0, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn multi-shot mode off" << endl;
        return cur_status;
    }
    
    // Adjust the Gain feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_GAIN, 2048);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain value to 2048" << endl;
        return cur_status;
    }

    // Adjust the Shutter feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHUTTER, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SHUTTER, 2600);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter value to 2600" << endl;
        return cur_status;
    }
    
    // Adjust the Brightness
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_BRIGHTNESS, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brightness to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_BRIGHTNESS, 0);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brighness value to 0" << endl;
        return cur_status;
    }
    
    // Adjust the Auto-Exposure
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Auto-Exposure to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_EXPOSURE, 435);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Auto-Exposure value to 435" << endl;
        return cur_status;
    }

    // Setup the capture
    cur_status=dc1394_capture_setup(ap_camera, a_dma_channels , DC1394_CAPTURE_FLAGS_DEFAULT);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to setup camera capture" << endl;
        return cur_status;
    }
    return DC1394_SUCCESS;
*/
    dc1394error_t cur_status;
    using namespace std;
    
    // Turn the camera on
    cur_status=dc1394_set_camera_power(ap_camera, DC1394_ON);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to power up the camera" << endl;
        return cur_status;
    }
    
    // Isochronous speed of the camera
    cur_status=dc1394_video_set_iso_speed(ap_camera, DC1394_ISO_SPEED_400);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the isochronous speed of the camera" << endl;
        return cur_status;
    }

    // Set the operation mode to 1394A (instead of 1394B)    
    cur_status= dc1394_video_set_operation_mode(ap_camera, DC1394_OPERATION_MODE_LEGACY);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the operation mode of the camera" << endl;
        return cur_status;
    }
    
    // Set mode
    cur_status=dc1394_video_set_mode(ap_camera, DC1394_VIDEO_MODE_1024x768_MONO8);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the mode of the camera" << endl;
        return cur_status;
    }

    // Set framerate
    cur_status=dc1394_video_set_framerate(ap_camera, DC1394_FRAMERATE_1_875); // DC1394_FRAMERATE_7_5,DC1394_FRAMERATE_15
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the framerate of the camera" << endl;
        return cur_status;
    }
    
    // Turn one-shot off
    cur_status=dc1394_video_set_one_shot(ap_camera, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn one-shot mode off" << endl;
        return cur_status;
    }

    // Turn multi-shot off
    cur_status=dc1394_video_set_multi_shot(ap_camera, 0, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn multi-shot mode off" << endl;
        return cur_status;
    }
    
    // Adjust the Brightness
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_BRIGHTNESS, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brightness to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_BRIGHTNESS, reg_struct.brightness_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brighness value to " << reg_struct.brightness_val << endl;
        return cur_status;
    }

    // Adjust the Auto-Exposure
    if (reg_struct.autoexposure_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Auto-Exposure off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Auto-Exposure on" << endl;
            return cur_status;
        }

        if (reg_struct.autoexposure_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_EXPOSURE, reg_struct.autoexposure_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure value to " << reg_struct.autoexposure_val << endl;
                return cur_status;
            }
        }
    }    
    
    // Adjust the White-balance
    if (reg_struct.whitebalance_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set White Balance off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set White Balance on" << endl;
            return cur_status;
        }

        // Mode is set to manual
        cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_FEATURE_MODE_MANUAL);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set White Balance to Manual" << endl;
            return cur_status;
        }
        cur_status=dc1394_feature_whitebalance_set_value(ap_camera, reg_struct.blueu_val, reg_struct.redv_val);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Blue/U value to " << reg_struct.blueu_val << " and Red/V value to " << reg_struct.redv_val << endl;
            return cur_status;
        }
    }    

    // Adjust the Shutter feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHUTTER, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SHUTTER, reg_struct.shutter_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter value to 2600" << endl;
        return cur_status;
    }    
    
    // Adjust the Gain feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_GAIN, reg_struct.gain_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain value to 2048" << endl;
        return cur_status;
    }

    cur_status=dc1394_video_specify_iso_channel(ap_camera, iso_channel);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set ISO CHANNEL to " << iso_channel << endl;
        return cur_status;
    }
    
    return DC1394_SUCCESS;
}

dc1394error_t CamClass::setDragonFly2CameraRegisters(dc1394camera_t *ap_camera, int iso_channel, DF2_REGS &reg_struct)
{
    dc1394error_t cur_status;
    using namespace std;
    
    // Turn the camera on
    cur_status=dc1394_set_camera_power(ap_camera, DC1394_ON);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to power up the camera" << endl;
        return cur_status;
    }
    
    // Isochronous speed of the camera
    cur_status=dc1394_video_set_iso_speed(ap_camera, DC1394_ISO_SPEED_400);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the isochronous speed of the camera" << endl;
        return cur_status;
    }

    // Set the operation mode to 1394A (instead of 1394B)    
    cur_status= dc1394_video_set_operation_mode(ap_camera, DC1394_OPERATION_MODE_LEGACY);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the operation mode of the camera" << endl;
        return cur_status;
    }
    
    // Set mode
    cur_status=dc1394_video_set_mode(ap_camera, DC1394_VIDEO_MODE_1024x768_RGB8);
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the mode of the camera" << endl;
        return cur_status;
    }

    // Set framerate
    cur_status=dc1394_video_set_framerate(ap_camera, DC1394_FRAMERATE_1_875); // DC1394_FRAMERATE_7_5,DC1394_FRAMERATE_15
    if (cur_status!=DC1394_SUCCESS)
    {
        cerr << "Unable to set the framerate of the camera" << endl;
        return cur_status;
    }
    
    // Turn one-shot off
    cur_status=dc1394_video_set_one_shot(ap_camera, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn one-shot mode off" << endl;
        return cur_status;
    }

    // Turn multi-shot off
    cur_status=dc1394_video_set_multi_shot(ap_camera, 0, DC1394_OFF);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to turn multi-shot mode off" << endl;
        return cur_status;
    }
    
    // Adjust the Brightness
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_BRIGHTNESS, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brightness to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_BRIGHTNESS, reg_struct.brightness_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Brighness value to " << reg_struct.brightness_val << endl;
        return cur_status;
    }

    // Adjust the Auto-Exposure
    if (reg_struct.autoexposure_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Auto-Exposure off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Auto-Exposure on" << endl;
            return cur_status;
        }

        if (reg_struct.autoexposure_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_EXPOSURE, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_EXPOSURE, reg_struct.autoexposure_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Auto-Exposure value to " << reg_struct.autoexposure_val << endl;
                return cur_status;
            }
        }
    }    
    
    // Adjust the Sharpness
    if (reg_struct.sharpness_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_SHARPNESS, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Sharpness off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_SHARPNESS, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Sharpness on" << endl;
            return cur_status;
        }

        if (reg_struct.sharpness_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHARPNESS, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Sharpness to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHARPNESS, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Sharpness to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SHARPNESS, reg_struct.sharpness_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Sharpness value to " << reg_struct.sharpness_val << endl;
                return cur_status;
            }
        }
    }    

    // Adjust the White-balance
    if (reg_struct.whitebalance_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set White Balance off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set White Balance on" << endl;
            return cur_status;
        }

        if (reg_struct.whitebalance_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set White Balance to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set White Balance to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_whitebalance_set_value(ap_camera, reg_struct.blueu_val, reg_struct.redv_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Blue/U value to " << reg_struct.blueu_val << " and Red/V value to " << reg_struct.redv_val << endl;
                return cur_status;
            }
        }
    }    

    // Adjust the Hue
    if (reg_struct.hue_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_HUE, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Hue off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_HUE, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Hue on" << endl;
            return cur_status;
        }

        if (reg_struct.hue_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_HUE, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Hue to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_HUE, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Hue to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_HUE, reg_struct.hue_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Hue value to " << reg_struct.hue_val << endl;
                return cur_status;
            }
        }
    }    

    // Adjust the Saturation
    if (reg_struct.saturation_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_SATURATION, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Saturation off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_SATURATION, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Saturation on" << endl;
            return cur_status;
        }

        if (reg_struct.saturation_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SATURATION, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Saturation to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SATURATION, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Saturation to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SATURATION, reg_struct.saturation_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Saturation value to " << reg_struct.saturation_val << endl;
                return cur_status;
            }
        }
    }    

    // Adjust the Gamma
    if (reg_struct.gamma_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_GAMMA, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Gamma off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_GAMMA, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Gamma on" << endl;
            return cur_status;
        }

        // Mode is set to manual
        cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_GAMMA, DC1394_FEATURE_MODE_MANUAL);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Gamma to Manual" << endl;
            return cur_status;
        }
        cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_GAMMA, reg_struct.gamma_val);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Gamma value to " << reg_struct.gamma_val << endl;
            return cur_status;
        }
    }    

    // Adjust the Shutter feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_SHUTTER, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_SHUTTER, reg_struct.shutter_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Shutter value to 2600" << endl;
        return cur_status;
    }    
    
    // Adjust the Gain feature
    cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain to Manual" << endl;
        return cur_status;
    }
    cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_GAIN, reg_struct.gain_val);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set Gain value to 2048" << endl;
        return cur_status;
    }

    // Adjust the Iris
    if (reg_struct.iris_mode==-1) // feature off
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_IRIS, DC1394_OFF);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Iris off" << endl;
            return cur_status;
        }
    }
    else 
    {
        cur_status=dc1394_feature_set_power(ap_camera, DC1394_FEATURE_IRIS, DC1394_ON);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "Unable to set Iris on" << endl;
            return cur_status;
        }

        if (reg_struct.iris_mode==1) // Auto mode
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_IRIS, DC1394_FEATURE_MODE_AUTO);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Iris to Auto" << endl;
                return cur_status;
            }        
        }
        else // Mode is set to manual
        {
            cur_status=dc1394_feature_set_mode(ap_camera, DC1394_FEATURE_IRIS, DC1394_FEATURE_MODE_MANUAL);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Iris to Manual" << endl;
                return cur_status;
            }
            cur_status=dc1394_feature_set_value(ap_camera, DC1394_FEATURE_IRIS, reg_struct.iris_val);
            if (cur_status!=DC1394_SUCCESS) {
                cerr << "Unable to set Iris value to " << reg_struct.iris_val << endl;
                return cur_status;
            }
        }
    }    

    cur_status=dc1394_video_specify_iso_channel(ap_camera, iso_channel);
    if (cur_status!=DC1394_SUCCESS) {
        cerr << "Unable to set ISO CHANNEL to " << iso_channel << endl;
        return cur_status;
    }
    
    return DC1394_SUCCESS;
}

dc1394error_t CamClass::verifyCameraTransmission(dc1394camera_t *df_camera)
{
    dc1394switch_t status = DC1394_OFF;

    // DRAGON FLY CAMERA
    int i = 0;
    while( status == DC1394_OFF && i++ < 5 ) {
        usleep(50000);
        dc1394error_t ret_status=dc1394_video_get_transmission(df_camera, &status);
        if (ret_status!=DC1394_SUCCESS) {
            cerr << "unable to get transmision status" << endl;
            return ret_status;
        }
    }
    if( i == 5 ) {
        cerr << "Camera doesn't seem to want to turn on!" << endl;
        return DC1394_FAILURE;
    }

    return DC1394_SUCCESS;
}

dc1394error_t CamClass::empty_dma_channels(dc1394camera_t *ap_camera, int a_dma_channels)
{
    dc1394video_frame_t *frame;
    dc1394error_t cur_status;
    using namespace std;
    
    for (int temp1=1; temp1<=a_dma_channels; temp1++)        
    {
        cur_status=dc1394_capture_dequeue(ap_camera, DC1394_CAPTURE_POLICY_POLL, &frame);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "unable to capture a frame" << endl;
            return cur_status;
        }
        
        // Return the buffer to the DMA pool 
        cur_status=dc1394_capture_enqueue(ap_camera, frame);
        if (cur_status!=DC1394_SUCCESS) {
            cerr << "unable to capture a frame" << endl;
            return cur_status;
        }
    }
    return DC1394_SUCCESS;
} 


void CamClass::cleanup()
{
    for (unsigned int i=0; i<numCameras; i++)
    {
        dc1394_video_set_transmission(cameras[i], DC1394_OFF);
        dc1394_capture_stop(cameras[i]);
        dc1394_free_camera(cameras[i]);
    }
//    exit(1);
}

void    CamClass::readRegisterFileDF(char *regFilename,DF_REGS &reg_struct)
{
    ifstream    infile(regFilename, ifstream::in);

    // Brightness
    infile >> reg_struct.brightness_val;
    // Auto exposure
    infile >> reg_struct.autoexposure_mode;
    infile >> reg_struct.autoexposure_val;
    // White balance    
    infile >> reg_struct.whitebalance_mode;
    infile >> reg_struct.blueu_val;
    infile >> reg_struct.redv_val;
    // Shutter
    infile >> reg_struct.shutter_mode;
    infile >> reg_struct.shutter_val;
    // Gain  
    infile >> reg_struct.gain_mode;
    infile >> reg_struct.gain_val;

    infile.close();
    return;
}

void    CamClass::readRegisterFileDF2(char *regFilename,DF2_REGS &reg_struct)
{
    ifstream    infile(regFilename, ifstream::in);

    // Brightness
    infile >> reg_struct.brightness_val;
    // Auto exposure
    infile >> reg_struct.autoexposure_mode;
    infile >> reg_struct.autoexposure_val;
    // Sharpness
    infile >> reg_struct.sharpness_mode;
    infile >> reg_struct.sharpness_val;
    // White balance    
    infile >> reg_struct.whitebalance_mode;
    infile >> reg_struct.blueu_val;
    infile >> reg_struct.redv_val;
    // Hue 
    infile >> reg_struct.hue_mode;
    infile >> reg_struct.hue_val;
    // Saturation
    infile >> reg_struct.saturation_mode;
    infile >> reg_struct.saturation_val;
    // Gamma  
    infile >> reg_struct.gamma_mode;
    infile >> reg_struct.gamma_val;
    // Shutter
    infile >> reg_struct.shutter_mode;
    infile >> reg_struct.shutter_val;
    // Gain  
    infile >> reg_struct.gain_mode;
    infile >> reg_struct.gain_val;
    // Iris
    infile >> reg_struct.iris_mode;
    infile >> reg_struct.iris_val;

    infile.close();
    return;
}

void CamClass::printRegisterFile(DF2_REGS &reg_struct)
{
    // Brightness
    cout << "Brightness: Value" << reg_struct.brightness_val << endl;
    // Auto exposure
    cout << "Auto-exposure: Mode:" << reg_struct.autoexposure_mode << "\tValue:" << reg_struct.autoexposure_val << endl;
    // Sharpness
    cout << "Sharpness: Mode:" << reg_struct.sharpness_mode << "\tValue:" << reg_struct.sharpness_val << endl;
    // White balance    
    cout << "White balance: Mode:" << reg_struct.whitebalance_mode << "\tBlueU:" << reg_struct.blueu_val << "\tRedv:" << reg_struct.redv_val << endl;
    // Hue 
    cout << "Hue: Mode:"<< reg_struct.hue_mode << "\tValue:" << reg_struct.hue_val << endl;
    // Saturation
    cout << "Saturation: Mode:" << reg_struct.saturation_mode << "\tValue:" << reg_struct.saturation_val << endl;
    // Gamma  
    cout << "Gamma: Mode:" << reg_struct.gamma_mode << "\tValue:" << reg_struct.gamma_val << endl;
    // Shutter
    cout << "Shutter: Mode:" << reg_struct.shutter_mode << "\tValue:" << reg_struct.shutter_val << endl;
    // Gain  
    cout << "Gain: Mode:" << reg_struct.gain_mode << "\tValue:" << reg_struct.gain_val << endl;
    // Iris
    cout << "Iris: Mode:" << reg_struct.iris_mode << "\tValue:" << reg_struct.iris_val << endl << endl;
  
    return;
}


/*int main(int argc, const char **argv)
{    
    // Camera parameters    
    dc1394camera_t **cameras=NULL;
    dc1394camera_t *df_camera;
    dc1394camera_t *df2_camera;
    uint64_t       df_GUID; // The GUIDs of the cameras on the top row (0-> left, 1->right)
    uint64_t       df2_GUID; // The GUIDs of the cameras on the bottom row (0-> left, 1->right)
//    dfGUID=((uint64_t)0x08004602<<32)+0x0004008b;//(uint64_t)576537726707564683;//080046020004008b;
//    df2GUID=((uint64_t)0x08004602<<32)+0x00040088;//576537726707564680;//0800460200040088;
    df_GUID=((uint64_t)0x00b09d01<<32)+0x003f0618;//49712223525733912;//(uint64_t)0x00b09d01003f0618U;
//    dfGUID=((uint64_t)0x00b09d01<<32)+0x003f0619;//49712223525733913;//(uint64_t)0x00b09d01003f0619U;
    df2_GUID=((uint64_t)0x00b09d01<<32)+0x006d7142;//49712223525733913;//(uint64_t)0x00b09d01003f0619U;
//    df2GUID=((uint64_t)0x00b09d01<<32)+0x006d7145;//49712223525733913;//(uint64_t)0x00b09d01003f0619U;
    uint32_t numCameras;
    int      dma_channels=4;
    unsigned int df_width, df_height;
    unsigned int df2_width, df2_height;
    dc1394video_frame_t *df_frame, *df2_frame; // pointer to the structure that will contain our frames
  
    DF2_REGS df2_registers; // The structure that contains the values for all the dragonfly 2 registers
  
    dc1394color_filter_t bayerPattern;
    //////////////////////////////////////
    // Initialize some pointers to NULL //
    //////////////////////////////////////
    df_camera=NULL;
    df2_camera=NULL;

    //////////////////
    // Find cameras //
    //////////////////
    int err=dc1394_find_cameras(&cameras, &numCameras);

    if (err!=DC1394_SUCCESS && err != DC1394_NO_CAMERA) {
        cerr << "Unable to look for cameras" << endl << endl;
        cerr << "On Linux, please check" << endl;
        cerr << "  - if the kernel modules `ieee1394',`raw1394' and `ohci1394' are loaded" << endl;
        cerr << "  - if you have read/write access to /dev/raw1394" << endl << endl;
        exit(1);
    }
    
    //////////////////////////////////////////////////////////////////////////////
    // get the camera nodes and identify the dragonfly and dragonfly2 cameras   //
    //////////////////////////////////////////////////////////////////////////////
    if (numCameras<1) {
        cerr << "no cameras found" << endl;
        exit(1);
    }
    cout << "Found " << numCameras << " cameras on the 1394 bus" << endl;

    // Store the camera handles in the proper structure. To do that we should compare the GUID of the cameras
    for (unsigned int i=0; i<numCameras; i++)
    {
        if ((cameras[i]->euid_64)==df_GUID) df_camera=cameras[i];
        else if ((cameras[i]->euid_64)==df2_GUID) df2_camera=cameras[i];
    } 
    cout << "Found the cameras" << endl;
    //////////////////////////////////////////////
    // Read the register file and store the     //
    // values to the appropriate data structure //
    //////////////////////////////////////////////
    readRegisterFile("registerValues.txt",df2_registers);
    printRegisterFile(df2_registers);
    // Prepare the SONY XCD X-700 cameras
//    if (top_camera[0]!=NULL) setSonyCameraRegisters(top_camera[0], dma_channels);
//    if (top_camera[1]!=NULL) setSonyCameraRegisters(top_camera[1], dma_channels);
    
    // Prepare the DragonFly cameras
//    if (low_camera[0]!=NULL) setDragonFlyCameraRegisters(low_camera[0], dma_channels);
    if (df_camera!=NULL) setDragonFlyCameraRegisters(df_camera, dma_channels);

    if (df2_camera!=NULL) setDragonFly2CameraRegisters(df2_camera, dma_channels, df2_registers);

    // Obtain the Bayer Pattern in advance
    if ( getBayerTile( df_camera, &bayerPattern ) != DC1394_SUCCESS )
    {
        cerr << "Could not get bayer tile pattern from left camera" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
//    if ( getBayerTile( low_camera[1], &bayerPatternRight ) != DC1394_SUCCESS )
//    {
//        cerr << "Could not get bayer tile pattern from right camera" << endl;
//        cleanup_and_exit(cameras, numCameras);
//    }

    // Obtain the frame size for all the cameras in advance
    dc1394_get_image_size_from_video_mode( df_camera, DC1394_VIDEO_MODE_1024x768_MONO8, &df_width, &df_height );    
    cout << "Dragon Fly: Image size :" << df_width << "x" << df_height << endl;
    dc1394_get_image_size_from_video_mode( df2_camera, DC1394_VIDEO_MODE_1024x768_RGB8, &df2_width, &df2_height );    
    cout << "Dragon Fly2: Image size :" << df2_width << "x" << df2_height << endl;
    
    //////////////////////////////
    // start all the cameras    //
    //////////////////////////////
    if (df_camera!=NULL)
        if (dc1394_video_set_transmission(df_camera, DC1394_ON) !=DC1394_SUCCESS) {
            cerr << "unable to start dragon fly camera iso transmission" << endl;
            cleanup_and_exit(cameras, numCameras);
        }

    if (df2_camera!=NULL)
        if (dc1394_video_set_transmission(df2_camera, DC1394_ON) !=DC1394_SUCCESS) {
            cerr << "unable to start dragon fly2 camera iso transmission" << endl;
            cleanup_and_exit(cameras, numCameras);
        }
*/
/*    //////////////////////////////
    // Trigger the 2 cameras    //
    //////////////////////////////
    if (dc1394_software_trigger_set_power(df_camera, DC1394_ON)!=DC1394_SUCCESS) {
        cerr << "unable to set software trigger to on" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
    if (dc1394_software_trigger_set_power(df2_camera, DC1394_ON)!=DC1394_SUCCESS) {
        cerr << "unable to set software trigger to on" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
*/    
/*
    //////////////////////////////////////////////////////
    // Check if the cameras started to transmit data    //
    //////////////////////////////////////////////////////
    if (verifyCamerasTransmission(df_camera, df2_camera)!=DC1394_SUCCESS)
    {
        cerr << "Camera transmission function failed" << endl;
        cleanup_and_exit(cameras, numCameras);
    }        
*/
/*    // Empty all the DMA channels
    empty_dma_channels(df_camera, dma_channels); 
    empty_dma_channels(df2_camera, dma_channels);
*/
/*   
    //////////////////////////////////////////    
    // Obtain the frames from the 2 cameras //
    //////////////////////////////////////////
    if (dc1394_capture_dequeue(df_camera, DC1394_CAPTURE_POLICY_WAIT, &df_frame)!=DC1394_SUCCESS) {
        cerr << "unable to capture a frame" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
    if (dc1394_capture_dequeue(df2_camera, DC1394_CAPTURE_POLICY_WAIT, &df2_frame)!=DC1394_SUCCESS) {
        cerr << "unable to capture a frame" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
    
    //////////////////////////////////////
    // Write the images to the files    //
    //////////////////////////////////////
    writeRGBFrameToFile( "df2_im.ppm", df2_width, df2_height, df2_frame);
    writeRawFrameToFile( "df_im.ppm", df_width, df_height, df_frame, bayerPattern);

    //////////////////////////////////////////////
    // Return all the buffers to the DMA pool   //
    ////////////////////////////////////////////// 
    if (dc1394_capture_enqueue(df_camera, df_frame)!=DC1394_SUCCESS) {
        cerr << "unable to enqueue the DMA buffer" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
    if (dc1394_capture_enqueue(df2_camera, df2_frame)!=DC1394_SUCCESS) {
        cerr << "unable to enqueue the DMA buffer" << endl;
        cleanup_and_exit(cameras, numCameras);
    }
    
    cleanup_and_exit(cameras, numCameras);
    return 0;
}
*/
